#define Version "EncoderStepper4"

#include <LiquidCrystal_I2C.h>
LiquidCrystal_I2C lcd(0x3F,16,2);
#define Enable  2
#define Pulse   3
#define Dir     4
#define Home    7

// encoder definitions
#define ENCODER_A   A0
#define ENCODER_B   A1
#define ENCODER_INPUT_PORT PINC
#define ENCODER_BUTTON 5

long encoder;                               // encoder direction: CCW=-1; NC=0; CW=1
int steps = 0;
int step_size = 1;
long MotorPosition = 0;

void setup()
{ 
pinMode(Enable,OUTPUT);
pinMode(Pulse,OUTPUT);
pinMode(Dir,OUTPUT);
pinMode(ENCODER_A,INPUT_PULLUP);     // encoder leads
pinMode(ENCODER_B,INPUT_PULLUP);  
pinMode(ENCODER_BUTTON,INPUT_PULLUP);
pinMode(Home,INPUT_PULLUP);

digitalWrite(Enable,HIGH);          // low = off
digitalWrite(Pulse,HIGH);
digitalWrite(Dir,HIGH);

lcd.init();                         // initialize the lcd 
lcd.backlight();
delay(100);
lcd.print(Version);
lcd.setCursor(0,1);
lcd.print("POS= ");
lcd.print(MotorPosition);
lcd.setCursor(12,1);
lcd.print("x");
lcd.print(step_size);
}

void loop() 
{  if(digitalRead(Home)==LOW)       // zero position at home.
   { MotorPosition = 0;
   } 
  check_step_size();
    // check the encoder 
  encoder = read_encoder();         // returns +1/0/-1 (cw/nc/ccw)
  if (encoder != 0) 
  { steps = steps + (encoder * step_size); 
    MotorPosition = MotorPosition+steps;
  } 
  if (steps >0)
  {   digitalWrite(Dir,LOW);       // forward
      DoSteps();
  }
  if (steps <0)
  {   steps = abs(steps);
      digitalWrite(Dir,HIGH);       // reverse
      if(digitalRead(Home)==HIGH)   // No step back if it is home.
      { DoSteps();                  // Not yet, so step.
      }
  }
}

/******************************
 read encoder direction
*******************************/
char read_encoder() 
{ // definitions
  #define DELAY 100                               // number of polling-loops   
  static unsigned short integrator;               // encoder debounce
 
  /* the array values are determined by combining the previous and current A1,A0 
  Gray Code bit-patterns to form an array-index. +1 is assigned to all clockwise 
  (CW) indexes; -1 is assigned to all counter-clockwise (CCW) indexes; and 0 is 
  assigned to the remaining positions to indicate no movement*/   
  static char encoder[] = {0,-1,1,0,1,0,0,-1,-1,0,0,1,0,1,-1,0}; // SWAPPED PHASES
  //   static char encoder[] = {0,1,-1,0,-1,0,0,1,1,0,0,-1,0,-1,1,0};
  static unsigned char encoder_index = 0;
  
  static long count = 0;                         // CW rotation = +1; CCW rotation = -1
  static long this_reading;
  static long last_reading;
  char value;                                    // return value

  /* check for encoder rotation. The absolute "count" value changes as the encoder rotates.
  Typically there are four states (counts) per click, but often more due to contact bounce.
  Bounce doesn't matter so long as the "count" decreases" for CCW rotation; remains constant 
  when there is no rotation; and increases with CW rotation. Bounce can be completely 
  eliminated if we look for the "detent" code and integrate as shown below. */  
  encoder_index <<= 2;                           // shift previous bit-pattern left two places
  encoder_index |= (ENCODER_INPUT_PORT & 0x03);  // get current bit-pattern and combine with previous
  count += encoder[( encoder_index & 0x0f )];     
  
  // debounce encoder
  if ((ENCODER_INPUT_PORT & 0x03) == 3)         // encoder on an indent (3 decimal == 00000011 binary)
  { if (integrator < DELAY)                     
    {  integrator++;
    } 
  } else                                        // encoder not resting on indent
    { if (integrator > 0) 
      { integrator--;
      }
    }
  
  // prepare the "return" value
  if (integrator >= DELAY)                      // encoder deemed to be stationary
  { this_reading = count;
    if (this_reading < last_reading) value = -1;
    if (this_reading == last_reading) value = 0;
    if (this_reading > last_reading) value = 1;
    last_reading = this_reading;                 // both readings now hold current "count"
    integrator = 0;                              // reset integrator
  } else 
    { value = 0;
    }
  return(value);
}

/******************************
 Drive the stepper
*******************************/
void DoSteps ()
  { int i = 0;
    for (i=0; i<steps; i++)
    {  digitalWrite(Pulse,LOW);
       delayMicroseconds(100);
       digitalWrite(Pulse,HIGH);
       delay(1);
    }
    steps = 0;
    lcd.setCursor(0,1);
    lcd.print("POS=        ");
    lcd.setCursor(4,1);
    if (MotorPosition >-1)
    {  lcd.print(" ");
    }
    lcd.print(MotorPosition);
  }

/******************************
 check the step size
*******************************/
void check_step_size() 
{ // delay definitions
  #define DELAY 3000                             // DELAY = debounce(mS)*loops/second
  #define LONG_PUSH 20000                        // number of loops considered "long"
  
  // variables
  static boolean transition = false;             // valid during button push
  static unsigned short integrator;              // varies from 0 to MAXIMUM
  static unsigned short push_duration;           // determines step size/direction
  
  // look for first high-to-low transition
  if (!transition && (digitalRead(ENCODER_BUTTON) == LOW)) 
  { transition = true;
    integrator = 0;
    push_duration = 0;
  }
  
  // look for last low-to-high transition
  if (transition) 
  { if (push_duration < (LONG_PUSH*2))          // track push duration
    { push_duration++;
    }
      
    if (digitalRead(ENCODER_BUTTON) == HIGH)    // ENCODER_BUTTON is HIGH
    {   if (integrator < DELAY)          
        {integrator++;
        } 
        else                                    // ENCODER_BUTTON is LOW
        {   if  (integrator > 0) 
            { integrator--;
            }
        }
    }
   
    // update the step_size
    if (integrator >= DELAY)  
    {  transition = false;
      if (push_duration > LONG_PUSH)           // LONG PUSH: decrease step size
      {  step_size = step_size/10;
        if (step_size < 1)                     //
        {   step_size = 100;                   // wrap around    
        }        
      } 
      else                                   // SHORT PUSH: increase step size
      {  step_size = step_size*10;
        if (step_size > 100) 
        {   step_size = 1;                       // wrap around 
        }         
      }
 // display the step multiplier     
    lcd.setCursor(12,1);
    lcd.print("x   ");                     // update step-size   
    lcd.setCursor(13,1);
    lcd.print(step_size);   
    }
  }
}
